﻿using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
using System.Linq;
//using System.Threading.Tasks;
using System.Windows.Forms;

namespace FrequencyCounter_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		FrequencyCounter openCh = null;
		formCleaner cleaner;
		FrequencyCounter freqCounter = null;
		private ErrorEventBox errorBox;

		private DataGraphBox freqGraphBox, countGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(FrequencyCounter ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {
			filterTypeCombo.DataSource = typeof(FrequencyFilterType).ToList();
			filterTypeCombo.SelectedIndex = -1;

			timeoutTip.SetToolTip(setCutoffFreqBtn, "Sets the length of time the device waits for new data before calculating frequency.");

			freqGraphBox = new DataGraphBox(this, "Frequency");
			countGraphBox = new DataGraphBox(this, "Count");
			errorBox = new ErrorEventBox(this);
			freqCounter = openCh ?? openArgs.makeChannel<FrequencyCounter>();

			freqCounter.Attach += freqCounter_attach;
			freqCounter.Detach += freqCounter_detach;
			freqCounter.Error += freqCounter_error;
			freqCounter.CountChange += freqCounter_change;
			freqCounter.FrequencyChange += freqCounter_FrequencyChange;
			freqCounter.PropertyChange += FreqCounter_PropertyChange;

			try {
				freqCounter.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void FreqCounter_PropertyChange(object sender, PropertyChangeEventArgs e) {

			switch (e.PropertyName) {
			case "PowerSupply":
				powerSupplyCmb.SelectedValue = freqCounter.PowerSupply;
				break;
			case "InputMode":
				inputModeCombo.SelectedValue = freqCounter.InputMode;
				break;
			}
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			freqCounter.Attach -= freqCounter_attach;
			freqCounter.Detach -= freqCounter_detach;
			freqCounter.Error -= freqCounter_error;
			freqCounter.CountChange -= freqCounter_change;
			freqCounter.FrequencyChange -= freqCounter_FrequencyChange;
			freqCounter.PropertyChange -= FreqCounter_PropertyChange;
			freqCounter.Close();
		}

		void freqCounter_attach(object sender, Phidget22.Events.AttachEventArgs e) {

			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);
			FrequencyCounter attachedDevice = (FrequencyCounter)sender;

			freqGraphBox.SetTitle(DataGraphBox.GenerateTitle("Frequency", (Phidget)freqCounter));
			countGraphBox.SetTitle(DataGraphBox.GenerateTitle("Count", (Phidget)freqCounter));

			settingsBox.Visible = true;
			outputBox.Visible = true;

			try {
				cutoffFreqTxt.Text = freqCounter.FrequencyCutoff.ToString();
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.Value = attachedDevice.DataInterval;
				dataIntervalTrk.Visible = true;


				try {
					attachedDevice.FilterType = attachedDevice.FilterType;

					filterPanel.Visible = true;
					filterTypeCombo.SelectedIndex = 0;
					attachedDevice.FilterType = (FrequencyFilterType)Enum.Parse(typeof(FrequencyFilterType), filterTypeCombo.SelectedValue.ToString());
				} catch (PhidgetException ex) {
					if (ex.ErrorCode == ErrorCode.Unsupported) {
						filterPanel.Visible = false;
					} else
						errorBox.addMessage("Error setting filter type: " + ex.Message);
				}

				try {
					attachedDevice.Enabled = attachedDevice.Enabled;

					enabledChk.Visible = true;
					enabledChk.Checked = freqCounter.Enabled;
				} catch (PhidgetException ex) {
					if (ex.ErrorCode == ErrorCode.Unsupported) {
						enabledChk.Visible = false;
					} else
						errorBox.addMessage("Error enabling: " + ex.Message);
				}

				try {
					PowerSupply temp = attachedDevice.PowerSupply;

					Enum[] supportedPowerSupplies;
					PowerSupplyPanel.Visible = true;
					supportedPowerSupplies = new Enum[] {PowerSupply.Off,
														 PowerSupply.Volts_12,
														 PowerSupply.Volts_24};
					powerSupplyCmb.SelectedIndexChanged -= powerSupplyCmb_SelectedIndexChanged;
					powerSupplyCmb.DataSource = supportedPowerSupplies.ToList();
					powerSupplyCmb.SelectedValue = attachedDevice.PowerSupply;
					powerSupplyCmb.SelectedIndexChanged += powerSupplyCmb_SelectedIndexChanged;
				} catch (PhidgetException ex) {
					if (ex.ErrorCode == ErrorCode.Unsupported) {
						PowerSupplyPanel.Visible = false;
					} else
						errorBox.addMessage("Error initializing: " + ex.Message);
				}

				try {
					InputMode temp = attachedDevice.InputMode;

					Enum[] supportedInputModes;
					inputModeBox.Visible = true;
					supportedInputModes = new Enum[] {InputMode.NPN,
													InputMode.PNP};
					inputModeCombo.SelectedIndexChanged -= inputModeCombo_SelectedIndexChanged;
					inputModeCombo.DataSource = supportedInputModes.ToList();
					inputModeCombo.SelectedValue = attachedDevice.InputMode;
					inputModeCombo.SelectedIndexChanged += inputModeCombo_SelectedIndexChanged;
				} catch (PhidgetException ex) {
					if (ex.ErrorCode == ErrorCode.Unsupported) {
						inputModeBox.Visible = false;
					} else
						errorBox.addMessage("Error initializing: " + ex.Message);
				}

				/*switch (freqCounter.DeviceID) {
				case DeviceID.PN_DAQ1400:
					Enum[] supportedPowerSupplies;
					PowerSupplyPanel.Visible = true;
					supportedPowerSupplies = new Enum[] {PowerSupply.Off,
														 PowerSupply.Volts_12,
														 PowerSupply.Volts_24};
					powerSupplyCmb.SelectedIndexChanged -= powerSupplyCmb_SelectedIndexChanged;
					powerSupplyCmb.DataSource = supportedPowerSupplies.ToList();
					powerSupplyCmb.SelectedValue = attachedDevice.PowerSupply;
					powerSupplyCmb.SelectedIndexChanged += powerSupplyCmb_SelectedIndexChanged;

					Enum[] supportedInputModes;
					inputModeBox.Visible = true;
					supportedInputModes = new Enum[] {InputMode.NPN,
													InputMode.PNP};
					inputModeCombo.SelectedIndexChanged -= inputModeCombo_SelectedIndexChanged;
					inputModeCombo.DataSource = supportedInputModes.ToList();
					inputModeCombo.SelectedValue = attachedDevice.InputMode;
					inputModeCombo.SelectedIndexChanged += inputModeCombo_SelectedIndexChanged;
					break;
				default:
					errorBox.addMessage("Unknown Device, example should be updated.");
					break;
				}*/
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing: " + ex.Message);
			}

		}

		void freqCounter_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			dataIntervalTrk.Visible = false;
			dataIntervalTrk.TrueValue = -1;
			PowerSupplyPanel.Visible = false;
			inputModeBox.Visible = false;
		}

		void freqCounter_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void freqCounter_change(object sender, Phidget22.Events.FrequencyCounterCountChangeEventArgs e) {
			countTxt.Text = e.Counts.ToString();
			timeTxt.Text = (e.TimeChange).ToString();
			totalCntTxt.Text = freqCounter.Count.ToString();
			totalTimeTxt.Text = (freqCounter.TimeElapsed).ToString();
			countGraphBox.addData(e.Counts, freqCounter.DataInterval);
		}

		void freqCounter_FrequencyChange(object sender, FrequencyCounterFrequencyChangeEventArgs e) {
			frequencyTxt.Text = e.Frequency.ToString() + " Hz";
			freqGraphBox.addData(e.Frequency, freqCounter.DataInterval);
		}

		private void cutoffFreqTxt_KeyDown(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					double val;
					if (Double.TryParse(cutoffFreqTxt.Text, out val))
						freqCounter.FrequencyCutoff = 1 / (val / 1000);
					else
						errorBox.addMessage("Cutoff frequency must be a double");
					e.SuppressKeyPress = true;
				} catch (PhidgetException ex) {
					errorBox.addMessage("Error setting frequency cutoff: " + ex.Message);
				}
			}
		}

		private void setCutoffFreqBtn_Click(object sender, EventArgs e) {
			try {
				double val;
				if (Double.TryParse(cutoffFreqTxt.Text, out val))
					freqCounter.FrequencyCutoff = val;
				else {
					errorBox.addMessage("Cutoff frequency must be a double");
					cutoffFreqTxt.Text = "";
				}
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting frequency cutoff: " + ex.Message);
			}
		}

		private void enabledChk_CheckedChanged(object sender, EventArgs e) {
			try {
				freqCounter.Enabled = enabledChk.Checked;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting enabled: " + ex.Message);
			}
		}

		private void filterTypeCombo_SelectedIndexChanged_1(object sender, EventArgs e) {
			if (((ComboBox)sender).SelectedIndex == -1 || freqCounter == null)
				return;

			try {
				freqCounter.FilterType = (FrequencyFilterType)Enum.Parse(typeof(FrequencyFilterType), filterTypeCombo.SelectedValue.ToString());
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting filter type: " + ex.Message);
			}
		}


		private void resetBtn_Click(object sender, EventArgs e) {
			try {
				freqCounter.Reset();
				totalCntTxt.Text = freqCounter.Count.ToString();
				totalTimeTxt.Text = freqCounter.TimeElapsed.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error resetting count: " + ex.Message);
			}
		}

		private void powerSupplyCmb_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				freqCounter.PowerSupply = (PowerSupply)Enum.Parse(typeof(PowerSupply), ((ComboBox)sender).SelectedValue.ToString());
			} catch (Exception ex) {
				errorBox.addMessage("Error setting voltage range: " + ex.Message);
			}
		}

		private void inputModeCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				freqCounter.InputMode = (InputMode)Enum.Parse(typeof(InputMode), ((ComboBox)sender).SelectedValue.ToString());
			} catch (Exception ex) {
				errorBox.addMessage("Error setting input mode: " + ex.Message);
			}
		}

		private void updateGraphDataInterval() {
			try {
				freqGraphBox.DataInterval = (1000.0 / freqCounter.DataRate);
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}
		private void countGraphBtn_Click(object sender, EventArgs e) {
			if (!countGraphBox.Visible)
				countGraphBox.Show();
		}

		private void freqGraphBtn_Click(object sender, EventArgs e) {
			if (!freqGraphBox.Visible)
				freqGraphBox.Show();
		}

		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e) {
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e) {
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					try {
						double temp = dataIntervalTrk.Value;
						freqCounter.DataInterval = (int)temp;
						if (freqCounter.DataInterval < (int)temp) {
							while (freqCounter.DataInterval != (int)temp) {
								temp++;
								freqCounter.DataInterval = (int)temp;
							}
						} else {
							while (freqCounter.DataInterval != (int)temp) {
								temp--;
								freqCounter.DataInterval = (int)temp;
							}
						}

						dataIntervalTrk.TrueValue = freqCounter.DataInterval;
						dataIntervalTrk.Value = freqCounter.DataInterval;



						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					try {
						freqCounter.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = freqCounter.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = freqCounter.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					if (freqCounter.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = freqCounter.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = freqCounter.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}